#!/bin/bash
#
#VERSION=1
#CHANGES="Initial Version"
IMAGE_URL=`echo $1 |sed 's/\/*$//'`
BLOCKLIST="7 6 5"

FLASH_CONFIG_STORAGE=/usr/conf/flash_update
FLASH_LOG=/usr/conf/flash.log
FLASH_LOG_OLD=/usr/conf/flash.log.old
CHROOT_DIR=/tmp/mchroot/
FLASH_LOCK=/tmp/flash.lock

function check_blocksize {
	xblock=$1
	[[ ! -f /tmp/$block ]] && { error=1; return; }
	size=$(ls -l /tmp/$block|awk '{print $5}')
	
	error=0
	case $xblock in
		
		mtdblock8)
			if [ $size -ne 3801088 ]; then
				error=1
				echo "<br>$xblock size wrong: expected 3801088, got $size"
			fi
		;;
		mtdblock7)
			if [ $size -ne 8388608 ]; then
				error=1
				echo "<br>$xblock size wrong: expected 8388608, got $size"
			fi
		;;
		mtdblock6)
			if [ $size -ne 10485760 ]; then
				error=1
				echo "<br>$xblock size wrong: expected 10485760, got $size"
			fi
		;;
		mtdblock5)
			if [ $size -ne 6291456 ]; then
				error=1
				echo "<br>$xblock size wrong: expected 6291456, got $size"
			fi
		;;
		*)
			echo "<br>check_blocksize: unexpected parameter<br>"
			error=1
		;;
	esac
}

########################################################################################################################
########################################################################################################################

#check or failed attempt
if [[ -f $FLASH_LOCK ]]; then
	echo "<h2>Failed flash process detected! You need to reboot before starting another attempt!</h2><br>"
	echo "<h2><font color='red'>Cancelled firmware flashing!</font></h2><br>"
	exit 1
fi


#check syntax
echo $IMAGE_URL|grep '^http' &> /dev/null
if [[ $? != 0 ]]; then
	echo "<h3>$IMAGE_URL<br><br> is not a valid URL.</h3><br>"
	echo "<h2><font color='red'>Cancelled firmware flashing!</font></h2><br>"
	exit 1
fi

#check if we are in update mode
FWSTATE=`beroconf get root boot_fwupdate`
if [ x"$FWSTATE" != x1 ]; then
	echo "<h1><font color='red'>Device must be in update mode!</font></h1><br>"
	exit 1
fi


#test download all needed blocks and check their size
cd /tmp/
FAILED=0
echo "<h2> Checking availability and integrity of image files!</h2><br>"
for _block in $BLOCKLIST; do
	block=mtdblock$_block
# 	echo wget $IMAGE_URL/$block -O /tmp/$block "<br>"
	echo "checking: $IMAGE_URL/$block<br>"
	wget $IMAGE_URL/$block -O /tmp/$block &> /dev/null
	if [[ $? != 0 ]]; then
		echo "<h2><font color='red'>Downloading image</font></h2><br>"
		echo "<h3> $IMAGE_URL/$block </h3><br>"
		echo "<h2><font color='red'>failed!</font></h2><br>"
		echo "<h2><font color='brown'>Cancelled firmware flashing! Please check your network connectivity!</font></h2>"
		FAILED=1
	
	else
		#we could successfully downloads
		check_blocksize $block
		if [ $error != 0 ]; then
			echo "<h2><font color='red'>Downloaded image</font></h2><br>"
			echo "<h2> $IMAGE_URL/$block </h2><br>"
			echo "<h2><font color='red'>has wrong size!</font></h2><br>"
			echo "<h2><font color='brown'>Cancelled firmware flashing! Please check your image integrity!</font></h2>"
			FAILED=1
		fi
		
		wget $IMAGE_URL/$block.md5 -O /tmp/$block.md5sum.1 &> /dev/null
		if [[ $? = 0 ]]; then
			file_md5sum=`head -1 /tmp/$block.md5sum.1|sed 's/ .*//'`
			block_md5sum=`md5sum /tmp/$block|sed 's/ .*//'`
			
			if [ x"$file_md5sum" != x"$block_md5sum" ]; then
				echo "<h2><font color='red'>Downloaded image</font></h2><br>"
				echo "<h2> $IMAGE_URL/$block </h2><br>"
				echo "<h2><font color='red'>has wrong md5sum $file_md5sum!<br>Expected $block_md5sum!</font></h2><br>"
				echo "<h2><font color='brown'>Cancelled firmware flashing! Please check your image integrity!</font></h2><br>"
				FAILED=1
			fi
			echo "md5sum: $file_md5sum matches<br>"
		else
# 			echo "no md5sum check<br>"
			echo "<h2><font color='red'>Downloaded image</font></h2><br>"
			echo "<h2> $IMAGE_URL/$block </h2><br>"
			echo "<h2><font color='red'>has no md5sum file!</font></h2><br>"
			echo "<h2><font color='brown'>Cancelled firmware flashing! Please check your image integrity!</font></h2><br>"
			FAILED=1
		fi
		
	fi
	rm -f /tmp/$block
	[[ $FAILED = 1 ]] && exit 1
	echo "<br>Succesfully downloaded $block <br>------------------------------------------------------------------------<br><br>"
done



#stop the crond, we don't want any unforeseen actions
/etc/init.d/S*cron stop &> /dev/null
killall -9 crond &> /dev/null

# MAYBE TODO
# lighttpd, remotesupport.sh, netconfig server and sshd are still running
# this could be moved too into the chroot environment

echo "<h1><font color='red'>Do not turn off the device!</font></h1><br>"

#####################################################################
#####################################################################
## here the actual flashing starts
## all necessary binaries are copied into a folder under tmp
## then the script chroots into this folder
## fetches the image for mtdblock7 and writes it to the flash
## then it does it for mtdblock6 and mtdblock5 too and reboots

#####################################################################
#####################################################################


#set flag that we started the flash process
#it prevents a second call to this script
touch $FLASH_LOCK

#prepare the chroot environment
if [ ! -d $CHROOT_DIR/ ]; then
	mkdir $CHROOT_DIR
	mkdir $CHROOT_DIR/dev/
	mkdir $CHROOT_DIR/proc/
	mkdir $CHROOT_DIR/sys/
	mkdir -p $CHROOT_DIR/usr/conf/
	mkdir $CHROOT_DIR/tmp/
	mkdir -p $CHROOT_DIR/extra/
	ln -s ../bin/busybox $CHROOT_DIR/extra/md5sum
	ln -s ../bin/busybox $CHROOT_DIR/extra/wget
	ln -s ../bin/busybox $CHROOT_DIR/extra/head
	ln -s ../bin/busybox $CHROOT_DIR/extra/tee
fi

export PATH=/extra/:$PATH
if [ ! -f $CHROOT_DIR/bin/busybox ]; then
	cp -ar /bin $CHROOT_DIR/
	cp -ar /sbin $CHROOT_DIR/
	cp -ar /lib $CHROOT_DIR/
	cp -ar /etc $CHROOT_DIR/
	cp -ar /usr/bin/awk $CHROOT_DIR/bin/
fi

if [ ! -e $CHROOT_DIR/dev/mtdblock5 ]; then
	mount -o bind /dev/ $CHROOT_DIR/dev/
	mount -o bind /sys/ $CHROOT_DIR/sys/
	mount -o bind /proc/ $CHROOT_DIR/proc/
	
	mount -o bind /usr/conf/ $CHROOT_DIR/usr/conf/
fi

cd $CHROOT_DIR

rm -rf $FLASH_CONFIG_STORAGE.old
mkdir $FLASH_CONFIG_STORAGE

cp -p /etc/passwd $FLASH_CONFIG_STORAGE
cp -p /etc/shadow $FLASH_CONFIG_STORAGE
cp -p /etc/shadow- $FLASH_CONFIG_STORAGE
cp -p /etc/hostname.default $FLASH_CONFIG_STORAGE
cp -p /etc/hostname.default $FLASH_CONFIG_STORAGE
cp -rp /root $FLASH_CONFIG_STORAGE
#copy the original md5 sums to the new tmp
for i in $BLOCKLIST; do
	cp -p /tmp/mtdblock$i.md5sum.1 $CHROOT_DIR/tmp/
done

#prepare the chroot environment
export -f check_blocksize

[[ -f "$FLASH_LOG" ]] && { \mv "$FLASH_LOG" "$FLASH_LOG_OLD"; }
echo Starting flash process: `date` >> $FLASH_LOG

/usr/sbin/chroot $CHROOT_DIR /bin/bash << EOF
#########################################

cd /tmp/
error=0
echo "<br><br><h1><font color='blue'>Flashing now!</font></h1>";
echo "<br><br><h1><font color='blue'>Please wait for the process to finish.</font></h1><br><br>";


#actual download all needed blocks and write them to their partitions
for _block in $BLOCKLIST; do
	#we repeat it
	for attempt in 1 2 3; do
		block=mtdblock\$_block
		echo "==============================================================================================<br>" | tee -a $FLASH_LOG
		
		echo "<br>"
		echo "wget $IMAGE_URL/\$block -O /tmp/\$block <br>" | tee -a $FLASH_LOG
		wget $IMAGE_URL/\$block -O /tmp/\$block
		echo "<br>"
		
		check_blocksize \$block
		if [ \$error != 0 ]; then
			echo "fetching $IMAGE_URL/\$block failed <br>" | tee -a $FLASH_LOG
			exit 1
		fi
		
		#get the md5 file again
		echo "wget $IMAGE_URL/\$block.md5 -O /tmp/\$block.md5sum.2" | tee -a $FLASH_LOG
		echo "<br>"
		wget $IMAGE_URL/\$block.md5 -O /tmp/\$block.md5sum.2
		echo "<br>"
		
		file1_md5sum=\$(head -1 /tmp/\$block.md5sum.1|sed 's/ .*//')
		file2_md5sum=\$(head -1 /tmp/\$block.md5sum.2|sed 's/ .*//')
		block1_md5sum=\$(md5sum /tmp/\$block|sed 's/ .*//')
	
		echo file1_md5sum \$file1_md5sum >> $FLASH_LOG
		echo file2_md5sum \$file2_md5sum >> $FLASH_LOG
		echo block1_md5sum \$block1_md5sum >> $FLASH_LOG
	
		#check if its the same as in the first test
		if [[ \$file1_md5sum != \$file2_md5sum ]]; then
			echo "md5sum changed from first test<br>Cancelling process!<br>" | tee -a $FLASH_LOG
			exit 1
		fi
		
		#check if downloaded image matches
		if [[ \$block1_md5sum != \$file2_md5sum ]]; then
			echo "md5sum of image doesn't match!<br>Cancelling process!<br>" | tee -a $FLASH_LOG
			exit 1
		fi
		echo "<br>"
		
		echo "dd if=/tmp/\$block of=/dev/\$block bs=1024k <br>" | tee -a $FLASH_LOG
		time dd if=/tmp/\$block of=/dev/\$block bs=1024k
		echo "flashed \$block <br><br>" | tee -a $FLASH_LOG
		rm -f /tmp/\$block
		
		block2_md5sum=\$(md5sum /dev/\$block|sed 's/ .*//')
		echo block2_md5sum \$block1_md5sum >> $FLASH_LOG
		if [[ \$block2_md5sum = \$block2_md5sum ]];then
			echo "md5sum of written partition matches<br>" | tee -a $FLASH_LOG
			break
		else
			
			if [[ \$attempt = 3 ]]; then
				echo "md5sum mismatch of partition \$block<br>" | tee -a $FLASH_LOG
				echo "giving up, something went wrong<br>" | tee -a $FLASH_LOG
				echo "hope for the best!<br>" | tee -a $FLASH_LOG
				break
			else
				echo "md5sum mismatch of partition \$block in attempt \$attempt, repeating writing of partition<br>" | tee -a $FLASH_LOG
			fi
		fi
	done
done
echo '<meta http-equiv="refresh" content="120;url=../app/berogui/" />'
echo "Rebooting now.<br>"| tee -a $FLASH_LOG
echo "<br><h1><font color='red'>Finished Successfully!<br>Rebooting now!</font></h1><br>"

echo Ended flash process: \$(date) >> $FLASH_LOG
/sbin/reboot

EOF

echo "ddflash.sh ended <br>"
